import {
  AggregateID,
  IdResponse,
  RpcExceptionCode,
  RpcValidationPipe,
} from '@mobicoop/ddd-library';
import { Controller, UsePipes } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { UpdateUserRequestDto } from './dtos/update-user.request.dto';
import { UpdateUserCommand } from '@modules/user/core/application/commands/update-user/update-user.command';
import {
  EmailAlreadyExistsException,
  PhoneAlreadyExistsException,
  UserAlreadyExistsException,
} from '@modules/user/core/domain/user.errors';

@UsePipes(
  new RpcValidationPipe({
    whitelist: true,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class UpdateUserGrpcController {
  constructor(private readonly commandBus: CommandBus) {}

  @GrpcMethod('UserService', 'Update')
  async updateUser(data: UpdateUserRequestDto): Promise<IdResponse> {
    try {
      const aggregateID: AggregateID = await this.commandBus.execute(
        new UpdateUserCommand({
          id: data.id,
          firstName: data.firstName,
          lastName: data.lastName,
          email: data.email,
          phone: data.phone,
        }),
      );
      return new IdResponse(aggregateID);
    } catch (error: any) {
      if (
        error instanceof UserAlreadyExistsException ||
        error instanceof EmailAlreadyExistsException ||
        error instanceof PhoneAlreadyExistsException
      )
        throw new RpcException({
          code: RpcExceptionCode.ALREADY_EXISTS,
          message: error.message,
        });
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: error.message,
      });
    }
  }
}
