import { Test, TestingModule } from '@nestjs/testing';
import { AggregateID, UniqueConstraintException } from '@mobicoop/ddd-library';
import { ConflictException } from '@mobicoop/ddd-library';
import { CreateUserRequestDto } from '@modules/user/interface/grpc-controllers/dtos/create-user.request.dto';
import { CreateUserService } from '@modules/user/core/application/commands/create-user/create-user.service';
import { USER_REPOSITORY } from '@modules/user/user.di-tokens';
import { UserEntity } from '@modules/user/core/domain/user.entity';
import { CreateUserCommand } from '@modules/user/core/application/commands/create-user/create-user.command';
import {
  EmailAlreadyExistsException,
  PhoneAlreadyExistsException,
  UserAlreadyExistsException,
} from '@modules/user/core/domain/user.errors';

const createUserRequest: CreateUserRequestDto = {
  firstName: 'John',
  lastName: 'Doe',
  email: 'john.doe@email.com',
  phone: '+33611223344',
};

const mockUserRepository = {
  insert: jest
    .fn()
    .mockImplementationOnce(() => ({}))
    .mockImplementationOnce(() => {
      throw new Error();
    })
    .mockImplementationOnce(() => {
      throw new ConflictException('User already exists');
    })
    .mockImplementationOnce(() => {
      throw new UniqueConstraintException('email already exists');
    })
    .mockImplementationOnce(() => {
      throw new UniqueConstraintException('phone already exists');
    }),
};

describe('create-user.service', () => {
  let createUserService: CreateUserService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: USER_REPOSITORY,
          useValue: mockUserRepository,
        },
        CreateUserService,
      ],
    }).compile();

    createUserService = module.get<CreateUserService>(CreateUserService);
  });

  it('should be defined', () => {
    expect(createUserService).toBeDefined();
  });

  describe('execution', () => {
    const createUserCommand = new CreateUserCommand(createUserRequest);
    it('should create a new user', async () => {
      UserEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      const result: AggregateID = await createUserService.execute(
        createUserCommand,
      );
      expect(result).toBe('047a6ecf-23d4-4d3e-877c-3225d560a8da');
    });
    it('should throw an error if something bad happens', async () => {
      UserEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      await expect(
        createUserService.execute(createUserCommand),
      ).rejects.toBeInstanceOf(Error);
    });
    it('should throw an exception if User already exists', async () => {
      UserEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      await expect(
        createUserService.execute(createUserCommand),
      ).rejects.toBeInstanceOf(UserAlreadyExistsException);
    });
    it('should throw an exception if Email already exists', async () => {
      UserEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      await expect(
        createUserService.execute(createUserCommand),
      ).rejects.toBeInstanceOf(EmailAlreadyExistsException);
    });
    it('should throw an exception if Phone already exists', async () => {
      UserEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      await expect(
        createUserService.execute(createUserCommand),
      ).rejects.toBeInstanceOf(PhoneAlreadyExistsException);
    });
  });
});
