import { RedisClientOptions } from '@liaoliaots/nestjs-redis';
import { Module, Provider } from '@nestjs/common';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { CqrsModule } from '@nestjs/cqrs';
import { redisStore } from 'cache-manager-ioredis-yet';
import { CacheModule } from '@nestjs/cache-manager';
import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';
import { CreateUserGrpcController } from './interface/grpc-controllers/create-user.grpc.controller';
import { UpdateUserGrpcController } from './interface/grpc-controllers/update-user.grpc.controller';
import { CreateUserService } from './core/application/commands/create-user/create-user.service';
import { UpdateUserService } from './core/application/commands/update-user/update-user.service';
import { USER_MESSAGE_PUBLISHER, USER_REPOSITORY } from './user.di-tokens';
import { UserRepository } from './infrastructure/user.repository';
import { UserMapper } from './user.mapper';
import { PrismaService } from './infrastructure/prisma.service';
import { DeleteUserGrpcController } from './interface/grpc-controllers/delete-user.grpc.controller';
import { FindUserByIdGrpcController } from './interface/grpc-controllers/find-user-by-id.grpc.controller';
import { DeleteUserService } from './core/application/commands/delete-user/delete-user.service';
import { FindUserByIdQueryHandler } from './core/application/queries/find-user-by-id/find-user-by-id.query-handler';
import { PublishMessageWhenUserIsCreatedDomainEventHandler } from './core/application/event-handlers/publish-message-when-user-is-created.domain-event-handler';
import { PublishMessageWhenUserIsUpdatedDomainEventHandler } from './core/application/event-handlers/publish-message-when-user-is-updated.domain-event-handler';
import { PublishMessageWhenUserIsDeletedDomainEventHandler } from './core/application/event-handlers/publish-message-when-user-is-deleted.domain-event-handler';

const imports = [
  CqrsModule,
  CacheModule.registerAsync<RedisClientOptions>({
    imports: [ConfigModule],
    useFactory: async (configService: ConfigService) => ({
      store: await redisStore({
        host: configService.get<string>('REDIS_HOST'),
        port: configService.get<number>('REDIS_PORT'),
        password: configService.get<string>('REDIS_PASSWORD'),
        ttl: configService.get('CACHE_TTL'),
      }),
    }),
    inject: [ConfigService],
  }),
];

const grpcControllers = [
  CreateUserGrpcController,
  UpdateUserGrpcController,
  DeleteUserGrpcController,
  FindUserByIdGrpcController,
];

const eventHandlers: Provider[] = [
  PublishMessageWhenUserIsCreatedDomainEventHandler,
  PublishMessageWhenUserIsUpdatedDomainEventHandler,
  PublishMessageWhenUserIsDeletedDomainEventHandler,
];

const commandHandlers: Provider[] = [
  CreateUserService,
  UpdateUserService,
  DeleteUserService,
];

const queryHandlers: Provider[] = [FindUserByIdQueryHandler];

const mappers: Provider[] = [UserMapper];

const repositories: Provider[] = [
  {
    provide: USER_REPOSITORY,
    useClass: UserRepository,
  },
];

const messagePublishers: Provider[] = [
  {
    provide: USER_MESSAGE_PUBLISHER,
    useExisting: MessageBrokerPublisher,
  },
];

const orms: Provider[] = [PrismaService];

@Module({
  imports,
  controllers: [...grpcControllers],
  providers: [
    ...eventHandlers,
    ...commandHandlers,
    ...queryHandlers,
    ...mappers,
    ...repositories,
    ...messagePublishers,
    ...orms,
  ],
  exports: [PrismaService, UserMapper, USER_REPOSITORY],
})
export class UserModule {}
